@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.4.2';
+our $VERSION = '0.1';
 
 =head1 NAME
 
@@ -101,6 +101,7 @@ use Carp;
 use Cwd qw(abs_path);
 use File::Spec::Functions qw(catfile);
 use File::Path qw(make_path remove_tree);
+use Time::HiRes qw(sleep);
 use File::Basename;
 
 use DBI;
@@ -135,9 +136,8 @@ sub init {
     my $dir   = shift;
 
     croak "Must specify a directory." unless $dir;
-    make_path($dir) unless -d $dir;
     $dir = abs_path $dir;
-    croak "Could not get absolute path: $!" unless $dir;
+    make_path($dir) unless -d $dir;
 
     my $self = bless {
         dir         => $dir,
@@ -177,11 +177,10 @@ C<OPTIONS> is a hashref that can contain the following options:
 A subroutine used to initialize the directory in the event that it gets
 created (although if the directory already exists when C<get_or_add> is called,
 this won't be called). This is called with the absolute path to the directory
-as the first arguemnt and the key name as the second argument.
-An exclusive lock is active on the directory for the duration of the function.
-If the function dies, then the entire call to C<get_or_add> will croak and the
-directory will not be created. If this isn't specified, an empty directory
-is created. (B<default:> undef)
+as the first, and only argument, and an exclusive lock is active on the
+directory for the duration of the function. If the function dies, then the
+entire call to C<get_or_add> will croak and the directory will not be created.
+If this isn't specified, an empty directory is created. (B<default:> undef)
 
 =item * B<lock_sh> I<(scalar ref)>
 
@@ -238,15 +237,12 @@ sub get_or_add {
     }
     else { $lockmode = UN }
 
-    my $dbh = $self->_new_connection;
-
-    my $listing_lock;
-    if (Store::Directories::Lock::_get_lock_mode($dbh, '__LISTING__') != EX) {
-        $listing_lock = $self->lock_ex('__LISTING__');
-        croak "Could not get lock on store listing." unless $listing_lock;
-    }
+    # The __LISTING__ lock ensures that directories cannot be added or removed
+    # while we're wokring.
+    my $listing_lock = $self->lock_ex('__LISTING__');
 
     # Get directory if it exists
+    my $dbh = $self->_new_connection;
     my $sth = $dbh->prepare("SELECT dir FROM entry WHERE slug = ?");
     $sth->execute($key) or  croak "Failed to fetch entries: ".$dbh->errstr;
     my $entry = $sth->fetch;
@@ -256,9 +252,8 @@ sub get_or_add {
     if ($entry) {
         # Create a lock if the user requested one
         if ($lockmode) {
-            $listing_lock->DESTROY if $listing_lock;
+            $listing_lock->DESTROY;
             $$lockref = Store::Directories::Lock->new($self, $key, $lockmode);
-            croak "Could not get requested lock on key '$key'." unless $$lockref;
         }
         return catfile($self->{dir}, $entry->[0]);
     }
@@ -275,25 +270,24 @@ sub get_or_add {
             # (if we're going to call an init function, we need it to be
             # an exclusive lock first, regardless of what the user asked for)
             my $mode = defined $opts->{init} ? EX : $lockmode;
-            Store::Directories::Lock::_add_lock_noblock($dbh, $key, $mode);
-            $lock = Store::Directories::Lock->_new_raw($self, $key, $$, $mode);
+            Store::Directories::Lock::_add_lock_noblock($dbh, $key, EX);
+            $lock = Store::Directories::Lock->_new_raw($self, $key, $$, EX);
         }
     });
-    $listing_lock->DESTROY if $listing_lock;
+    $listing_lock->DESTROY;
 
     # Use init function to create directory contents;
     if ($opts->{init}) {
         eval {
             use sigtrap qw(die normal-signals);
             die "'init' must be a subroutine ref" unless ref $opts->{init} eq 'CODE';
-            $opts->{init}->($dir, $key);
+            $opts->{init}->($dir);
             1;
         } or do {
             # Cleanup if callback failed
             my $errmsg = "Function call to initialize directory died. $@";
             {
                 my $temp_lock = $self->lock_ex('__LISTING__');
-                croak "Could not get lock on store listing." unless $temp_lock;
                 $self->_remove_directory_from_db($dbh, $key);
             }
             eval {
@@ -316,7 +310,7 @@ sub get_or_add {
     return $dir;
 }
 
-=item * B<lock_sh> I<KEY>, I<[NOBLOCK]>
+=item * B<lock_sh> I<KEY>
 
 Create and return a new I<shared> lock for the given key. This asserts that
 no other process can modify the corresponding entry until this lock goes
@@ -324,34 +318,33 @@ out-of-scope.
 
 This blocks until the lock can be obtained. So it will wait for any processes
 that already have an exclusive lock on this key to release their locks before
-returning. But if C<NOBLOCK> is true, then this will not block but may return
-undef if the lock couldn't be obtained.
+returning.
 
 This will croak if this process already has a lock (either kind) on this key,
 or if the key does not exist in the store.
 =cut
 sub lock_sh {
-    my ($self, $key, $noblock) = @_;
-    return Store::Directories::Lock->new($self, $key, SH, $noblock);
+    my $self = shift;
+    my $key  = shift;
+    return Store::Directories::Lock->new($self, $key, SH);
 }
 
-=item * B<lock_ex> I<KEY> I<[NOBLOCK]>
+=item * B<lock_ex> I<KEY>
 
 Create and return a new I<exclusive> lock for the given key. This asserts that
 no other process can read the corresponding entry until this lock goes
 out-of-scope.
 
 This blocks until the lock can be obtained. So it will wait for any processes
-that have locks on this key to release them before returning. But if C<NOBLOCK>
-is true, then this will not block but may return undef if the lock couldn't
-be obtained.
+that have locks on this key to release them before returning.
 
 This will croak if this process already has a lock (either kind) on this key,
 or if the key does not exist in the store.
 =cut
 sub lock_ex {
-    my ($self, $key, $noblock) = @_;
-    return Store::Directories::Lock->new($self, $key, EX, $noblock);
+    my $self = shift;
+    my $key  = shift;
+    return Store::Directories::Lock->new($self, $key, EX);
 }
 
 =item * B<remove> I<KEY> I<[SUB]>
@@ -359,8 +352,8 @@ sub lock_ex {
 Remove the directory with the given key from the store. You I<MUST> have an
 exclusive lock already on the directory before calling this. C<SUB> is a
 subroutine ref which, if specified, will be called immediately before deleting
-the directory. C<SUB> is called with the path to the directory as the first
-argument and the key for the directory as the second argument.
+the directory. C<SUB> is called with the path to the directory as the first and
+only argument.
 
 If an error occurs removing the directory from disk, (from C<SUB> failing,
 or otherwise), then the directory will still be removed from the store's index
@@ -371,11 +364,7 @@ sub remove {
     my ($self, $key, $callback) = @_;
     my $dbh = $self->_new_connection;
 
-    my $listing_lock;
-    if (Store::Directories::Lock::_get_lock_mode($dbh, '__LISTING__') != EX) {
-        $listing_lock = $self->lock_ex('__LISTING__');
-        croak "Could not get lock on store listing." unless $listing_lock;
-    }
+    my $listing_lock = $self->lock_ex('__LISTING__');
 
     # First check that the directory exists
     my $sth = $dbh->prepare("SELECT dir FROM entry WHERE slug = ?");
@@ -389,7 +378,7 @@ sub remove {
         return 1;
     }
     my $dir  = $entry->[0];
-    my $path = catfile($self->{dir}, $dir);
+    my $path = abs_path( catfile($self->{dir}, $dir) );
 
     # Check that we have an exclusive lock
     my $current_lock = Store::Directories::Lock::_get_lock_mode($dbh, $key);
@@ -400,13 +389,13 @@ sub remove {
 
     # Remove from database
     $self->_remove_directory_from_db($dbh, $key);
-    $listing_lock->DESTROY if $listing_lock;
+    $listing_lock->DESTROY;
 
     # Attempt to remove directory from disk
     eval {
         if ($callback) {
             die "'SUB' must be a subroutine ref" unless ref $callback eq 'CODE';
-            $callback->($path, $key);
+            $callback->($path);
         }
         $self->_remove_directory_from_disk($dir);
         1;
@@ -466,134 +455,8 @@ END
     return \%entries;
 }
 
-=item * B<get_in_dir> I<KEY>, I<SUB> I<[INIT]>
-
-Get a shared lock for the directory with key, C<KEY>, then execute the
-subroutine reference, C<SUB> (calling with the absolute path to the directory
-as the first argument and the key as the second argument).
-Returns whatever C<SUB> returns. Essentially, this is just a convenient
-shortcut for something like this:
-
-    my $dir  = $store->get_or_add('foo');
-    my $lock = $store->lock_sh('foo');
-    my $val = do_whatever($dir, 'foo');
-
-    # shortcut
-    my $val = $store->get_in_dir('foo', \&do_whatever);
-
-Naturally, your C<SUB> subroutine shouldn't modify the contents of the
-directory or else you'll be violating the trust that L<Store::Directories>
-(and other processes!) place in you.
-
-The optional C<INIT> argument is a subroutine used to initialize the directory
-in the event it doesn't yet exist when this is called. (Same semantics as the
-C<init> option to C<get_or_add>).
-=cut
-sub get_in_dir {
-    my ($self, $key, $sub, $init) = @_;
-
-    croak "'SUB' must be a subroutine ref." unless (ref $sub eq 'CODE');
-
-    my $lock;
-    return $sub->( $self->get_or_add(
-        $key, {init=>$init,lock_sh=>\$lock}
-    ), $key );
-}
-
-=item * B<run_in_dir> I<KEY>, I<SUB> I<[INIT]>
-
-Get an exclusive lock for the directory with key, C<KEY>, then execute the
-subroutine reference, C<SUB> (calling with the absolute path to the directory
-as the first argument and the key as the second argument). Returns whatever
-C<SUB> returns. Essentially, this is just a convenient shortcut for
-something like this:
-
-    my $dir  = $store->get_or_add('foo');
-    my $lock = $store->lock_ex('foo');
-    my $val = do_whatever($dir, 'foo');
-
-    # shortcut
-    my $val = $store->run_in_dir('foo', \&do_whatever);
-
-Unlike C<get_in_dir>, your C<SUB> subroutine is allowed to modify (or even
-delete!) the directory and its contents.
-
-The optional C<INIT> argument is a subroutine used to initialize the directory
-in the event it doesn't yet exist when this is called. (Same semantics as the
-C<init> option to C<get_or_add>).
-=cut
-sub run_in_dir {
-    my ($self, $key, $sub, $init) = @_;
-
-    croak "'SUB' must be a subroutine ref." unless (ref $sub eq 'CODE');
-
-    my $lock;
-    return $sub->( $self->get_or_add(
-        $key, {init=>$init,lock_ex=>\$lock}
-    ), $key );
-}
-
-=item * B<get_or_set> I<KEY>, I<GET>, I<SET> I<[INIT]>
-
-A combination of C<get_in_dir> and C<run_in_dir>. C<GET> and C<SET> are
-subroutine references. For the directory with key, C<KEY>, runs the C<GET>
-subroutine under a shared lock and returns whatever it returns. But if C<GET>
-returns C<undef>, then it will call C<SET> under an exclusive lock before
-trying C<GET> again. (If it returns C<undef> this time, then this method
-will just return C<undef>).
-
-Both subroutines are called with the absolute path to the directory as the
-first argument, and the key as the second argument.
-If any of them die, then this entire function will croak.
-
-This is useful when you have multiple processes that may want to perform some
-operation in the same directory, but you want to make sure that operation is
-only performed once. C<GET> can be made to return undef if it detects the
-operation has not been done yet, while C<SET> performs the operation.
-
-Be aware that C<GET> may actually get called up to three times. First, under
-the shared lock. And, if it returns C<undef>, then it will be called again
-immediately after upgrading to an exclusive lock (in case another process got
-to the exclusive lock first and already called C<SET> for us). If that's still
-C<undef>, then it will be called a third and final time.
-
-The optional C<INIT> argument is a subroutine used to initialize the directory
-in the event it doesn't yet exist when this is called. (Same semantics as the
-C<init> option to C<get_or_add>).
-=cut
-sub get_or_set {
-    my ($self, $key, $get, $set, $init) = @_;
-
-    croak "'GET' must be a subroutine ref." unless (ref $get eq 'CODE');
-    croak "'SET' must be a subroutine ref." unless (ref $set eq 'CODE');
-
-    my $sh;
-    my $dir = $self->get_or_add( $key, {init=>$init,lock_sh=>\$sh} );
-
-
-    my $retval = $get->($dir, $key);
-    return $retval if defined $retval;
-
-    # GET failed, so we need to call SET
-
-    # Get an exclusive lock
-    $sh->DESTROY;
-    my $ex = $self->lock_ex($key);
-    croak "Could not get exclusive lock." unless $ex;
-
-    # It's possible that another process beat us to the exclusive lock
-    # and just called SET for us, so we need to check GET again
-    $retval = $get->($dir, $key);
-    return $retval if defined $retval;
-
-    # Looks like *we're* the ones who're going to have to call SET after all
-    $set->($dir, $key);
-    return $get->($dir, $key);
-}
-
 =back
 
-
 =cut
 
 ########################
@@ -629,10 +492,6 @@ sub _new_connection {
         { %attrs, @_ }
     );
     croak "Failed to connect to database." unless $dbh;
-
-    $dbh->do("PRAGMA foreign_keys = ON")
-        or croak "Database error: ".$dbh->errstr;
-
     return $dbh;
 }
 
@@ -679,11 +538,7 @@ sub _add_directory {
 
     # Find a new UUID
     my $uuid;
-    my $dir;
-    do {
-        $uuid = $UUIDGEN->create_str();
-        $dir  = catfile($self->{dir}, $uuid);
-    } while ( exists $names->{$uuid} or -d $dir );
+    do { $uuid = $UUIDGEN->create_str() } while (exists $names->{$uuid});
 
     # Insert
     my $sth = $dbh->prepare("INSERT INTO entry (slug, dir) VALUES (? , ?);");
@@ -699,6 +554,7 @@ sub _add_directory {
     }
 
     # Create directory
+    my $dir = catfile($self->{dir}, $uuid);
     make_path($dir);
 
     return $dir;
